@@ -1,6 +1,5 @@
-/*	$Id$ */
 /*
- * Copyright (c) 2020--2021 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -608,7 +607,7 @@ static int
 rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	struct gemini *st, const struct lowdown_node *n)
 {
-	const struct lowdown_node	*child, *prev;
+	const struct lowdown_node	*child, *prev, *nn;
 	struct link			*l;
 	void				*pp;
 	struct lowdown_buf		*tmpbuf;
@@ -678,13 +677,31 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 		break;
 	case LOWDOWN_DEFINITION_TITLE:
 	case LOWDOWN_HRULE:
-	case LOWDOWN_LINEBREAK:
 	case LOWDOWN_LISTITEM:
 	case LOWDOWN_META:
 	case LOWDOWN_TABLE_ROW:
 		if (!rndr_buf_vspace(st, ob, 1))
 			return 0;
 		break;
+	case LOWDOWN_LINEBREAK:
+		if (!rndr_buf_vspace(st, ob, 1))
+			return 0;
+
+		/*
+		 * A linebreak is special if invoked within a
+		 * blockquote, because it means we want to output a
+		 * newline but also continue outputting the blockquote
+		 * marker.
+		 */
+
+		for (nn = n->parent; nn != NULL; nn = nn->parent)
+			if (nn->type == LOWDOWN_BLOCKQUOTE) {
+				if (!HBUF_PUTSL(ob, "> "))
+					return 0;
+				st->last_blank = 0;
+				break;
+			}
+		break;
 	case LOWDOWN_IMAGE:
 	case LOWDOWN_LINK:
 	case LOWDOWN_LINK_AUTO:
@@ -704,7 +721,7 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	case LOWDOWN_TABLE_BLOCK:
 	case LOWDOWN_BLOCKCODE:
 	case LOWDOWN_BLOCKHTML:
-		if (!HBUF_PUTSL(st->tmp, "```"))
+		if (!HBUF_PUTSL(st->tmp, "```\n"))
 			return 0;
 		if (!rndr_buf(st, ob, n, st->tmp))
 			return 0;
@@ -774,10 +791,14 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 			return 0;
 		st->last_blank = -1;
 		break;
-	case LOWDOWN_SUPERSCRIPT:
-		if (!HBUF_PUTSL(st->tmp, "^"))
+	case LOWDOWN_SUBSCRIPT:
+		if (!HBUF_PUTSL(st->tmp, "~") ||
+		    !rndr_buf(st, ob, n, st->tmp))
 			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp))
+		break;
+	case LOWDOWN_SUPERSCRIPT:
+		if (!HBUF_PUTSL(st->tmp, "^") ||
+		    !rndr_buf(st, ob, n, st->tmp))
 			return 0;
 		break;
 	default:
