@@ -1,9 +1,8 @@
-/*	$Id$ */
 /*
  * Copyright (c) 2008, Natacha Porté
  * Copyright (c) 2011, Vicent Martí
  * Copyright (c) 2014, Xavier Mendez, Devin Torres and the Hoedown authors
- * Copyright (c) 2016--2021 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -115,6 +114,15 @@ escape_attr(struct lowdown_buf *ob, const struct lowdown_buf *in)
 	return hesc_attr(ob, in->data, in->size);
 }
 
+static int
+newline(struct lowdown_buf *ob)
+{
+	if (ob->size > 0 && ob->data[ob->size - 1] != '\n' &&
+	    !hbuf_putc(ob, '\n'))
+		return 0;
+	return 1;
+}
+
 static int
 rndr_autolink(struct lowdown_buf *ob, 
 	const struct rndr_autolink *parm,
@@ -157,7 +165,7 @@ rndr_blockcode(struct lowdown_buf *ob,
 	const struct rndr_blockcode *parm,
 	const struct html *st)
 {
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 
 	if (parm->lang.size) {
@@ -211,7 +219,7 @@ rndr_definition(struct lowdown_buf *ob,
 	const struct lowdown_buf *content)
 {
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 	if (!HBUF_PUTSL(ob, "<dl>\n"))
 		return 0;
@@ -221,17 +229,65 @@ rndr_definition(struct lowdown_buf *ob,
 }
 
 static int
-rndr_blockquote(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+rndr_blockquote(const struct html *st,
+	const struct rndr_blockquote *param,
+	struct lowdown_buf *ob, const struct lowdown_buf *content)
 {
+	size_t	 i;
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
-	if (!HBUF_PUTSL(ob, "<blockquote>\n"))
+	if (param->type == BLOCKQUOTE_REGULAR || !(st->flags & 
+	    (LOWDOWN_HTML_CALLOUT_GFM|LOWDOWN_HTML_CALLOUT_MDN)))
+		return HBUF_PUTSL(ob, "<blockquote>\n") &&
+		    hbuf_putb(ob, content) &&
+		    HBUF_PUTSL(ob, "</blockquote>\n");
+
+	if (!HBUF_PUTSL(ob, "<div class=\""))
 		return 0;
-	if (!hbuf_putb(ob, content))
+
+	/* MDN/GFM style blockquotes. */
+
+	if (st->flags & LOWDOWN_HTML_CALLOUT_MDN) {
+		if (!hbuf_printf(ob, "%s",
+		    (param->admonition == ADMONITION_NOTE) ?
+		     "notecard note" :
+		    (param->admonition == ADMONITION_WARNING) ?
+		     "notecard warning" : "callout"))
+			return 0;
+	}
+	if (st->flags & LOWDOWN_HTML_CALLOUT_GFM) {
+		if (!hbuf_printf(ob, "%smarkdown-alert ",
+		    (st->flags & LOWDOWN_HTML_CALLOUT_MDN) ? " " : ""))
+			return 0;
+		if (!hbuf_printf(ob, "markdown-alert-%s",
+		    (param->admonition == ADMONITION_NOTE) ?
+		    "note" :
+		    (param->admonition == ADMONITION_WARNING) ?
+		    "warning" : "callout"))
+			return 0;
+	}
+
+	if (!HBUF_PUTSL(ob, "\">\n"))
 		return 0;
-	return HBUF_PUTSL(ob, "</blockquote>\n");
+
+	/*
+	 * Weird: content callouts have their initial callout type
+	 * removed.  Fortunately, this is hard-coded, so we know its
+	 * exact length.  Remove the callout type and any extra spacing
+	 * that follows it.
+	 */
+
+	if (param->admonition == ADMONITION_CALLOUT && content->size > 28) {
+		i = 28;
+		while (i < content->size && content->data[i] == ' ')
+			i++;
+		if (!HBUF_PUTSL(ob, "<p>") ||
+		    !hbuf_put(ob, content->data + i, content->size - i))
+			return 0;
+	} else if (!hbuf_putb(ob, content))
+		return 0;
+	return HBUF_PUTSL(ob, "</div>\n");
 }
 
 static int
@@ -320,7 +376,7 @@ rndr_header(struct lowdown_buf *ob, const struct lowdown_buf *content,
 	else if (level > 6)
 		level = 6;
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 	if (!hbuf_printf(ob, "<h%zu", level))
 		return 0;
@@ -405,7 +461,7 @@ rndr_list(struct lowdown_buf *ob,
 	const struct rndr_list *param)
 {
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 	if (param->flags & HLIST_FL_ORDERED) {
 		if (param->start > 1) {
@@ -514,7 +570,7 @@ rndr_paragraph(struct lowdown_buf *ob,
 	if (i == content->size)
 		return 1;
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 	if (!HBUF_PUTSL(ob, "<p>"))
 		return 0;
@@ -577,7 +633,7 @@ rndr_raw_block(struct lowdown_buf *ob,
 	if (org >= sz)
 		return 1;
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 
 	if (!hbuf_put(ob, param->text.data + org, sz - org))
@@ -601,7 +657,7 @@ static int
 rndr_hrule(struct lowdown_buf *ob)
 {
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 	return hbuf_puts(ob, "<hr/>\n");
 }
@@ -698,7 +754,7 @@ rndr_table(struct lowdown_buf *ob,
 	const struct lowdown_buf *content)
 {
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 	if (!HBUF_PUTSL(ob, "<table>\n"))
 		return 0;
@@ -712,7 +768,7 @@ rndr_table_header(struct lowdown_buf *ob,
 	const struct lowdown_buf *content)
 {
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (!newline(ob))
 		return 0;
 	if (!HBUF_PUTSL(ob, "<thead>\n"))
 		return 0;
@@ -726,7 +782,9 @@ rndr_table_body(struct lowdown_buf *ob,
 	const struct lowdown_buf *content)
 {
 
-	if (ob->size && !hbuf_putc(ob, '\n'))
+	if (content->size == 0)
+		return 1;
+	if (!newline(ob))
 		return 0;
 	if (!HBUF_PUTSL(ob, "<tbody>\n"))
 		return 0;
@@ -790,14 +848,15 @@ rndr_tablecell(struct lowdown_buf *ob,
 
 static int
 rndr_superscript(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+	const struct lowdown_buf *content, enum lowdown_rndrt type)
 {
+	const char	*elem;
 
-	if (!HBUF_PUTSL(ob, "<sup>"))
-		return 0;
-	if (!hbuf_putb(ob, content))
-		return 0;
-	return HBUF_PUTSL(ob, "</sup>");
+	elem = (type == LOWDOWN_SUPERSCRIPT) ? "sup" : "sub";
+
+	return hbuf_printf(ob, "<%s>", elem) &&
+	    hbuf_putb(ob, content) &&
+	    hbuf_printf(ob, "</%s>", elem);
 }
 
 static int
@@ -912,7 +971,7 @@ rndr_doc_footer(struct lowdown_buf *ob, const struct html *st)
 	 */
 
 	if (st->footsz > 0) {
-		if (ob->size && !hbuf_putc(ob, '\n'))
+		if (!newline(ob))
 			return 0;
 		if (!HBUF_PUTSL(ob,
 		    "<div class=\"footnotes\">\n<hr/>\n<ol>\n"))
@@ -970,7 +1029,7 @@ rndr_root(struct lowdown_buf *ob,
  */
 static int
 rndr_meta_multi(struct lowdown_buf *ob, const char *b, int href,
-	const char *starttag, const char *endtag)
+    const char *starttag, const char *endtag)
 {
 	const char	*start;
 	size_t		 sz, i, bsz;
@@ -999,14 +1058,10 @@ rndr_meta_multi(struct lowdown_buf *ob, const char *b, int href,
 
 		if (!hbuf_puts(ob, starttag))
 			return 0;
-		if (!HBUF_PUTSL(ob, "\""))
-			return 0;
 		if (!href && !hesc_attr(ob, start, sz))
 			return 0;
 		else if (href && !hesc_href(ob, start, sz))
 			return 0;
-		if (!HBUF_PUTSL(ob, "\""))
-			return 0;
 		if (!hbuf_puts(ob, endtag))
 			return 0;
 		if (!HBUF_PUTSL(ob, "\n"))
@@ -1106,34 +1161,45 @@ rndr_doc_header(struct lowdown_buf *ob,
 
 	/* Overrides. */
 
-	if (title == NULL)
-		title = "Untitled article";
 	if (rcsdate != NULL)
 		date = rcsdate;
 	if (rcsauthor != NULL)
 		author = rcsauthor;
 
 	if (!rndr_meta_multi(ob, affil, 0,
-	    "<meta name=\"creator\" content=", " />"))
+	    "<meta name=\"creator\" content=\"", "\" />"))
 		return 0;
-
 	if (!rndr_meta_multi(ob, author, 0,
-	    "<meta name=\"author\" content=", " />"))
+	    "<meta name=\"author\" content=\"", "\" />"))
 		return 0;
-
 	if (!rndr_meta_multi(ob, copy, 0,
-	    "<meta name=\"copyright\" content=", " />"))
+	    "<meta name=\"copyright\" content=\"", "\" />"))
 		return 0;
 
-	/*
-	 * FIXME: don't use "scheme" if the date isn't in the
-	 * appropriate format, or modify it depending upon the position
-	 * of the year?
-	 */
-
 	if (date != NULL) {
-		if (!hbuf_printf(ob, "<meta name="
-		    "\"date\" scheme=\"YYYY-MM-DD\" content=\""))
+		if (!HBUF_PUTSL(ob, "<meta name=\"date\" "))
+			return 0;
+
+		/*
+		 * Don't use "scheme" if the date isn't in the
+		 * appropriate format.
+		 */
+
+		if (strlen(date) == 10 &&
+		    isdigit((unsigned char)date[0]) &&
+		    isdigit((unsigned char)date[1]) &&
+		    isdigit((unsigned char)date[2]) &&
+		    isdigit((unsigned char)date[3]) &&
+		    date[4] == '-' &&
+		    isdigit((unsigned char)date[5]) &&
+		    isdigit((unsigned char)date[6]) &&
+		    date[7] == '-' &&
+		    isdigit((unsigned char)date[8]) &&
+		    isdigit((unsigned char)date[9]) &&
+		    !HBUF_PUTSL(ob, "scheme=\"YYYY-MM-DD\" "))
+			return 0;
+
+		if (!HBUF_PUTSL(ob, "content=\""))
 			return 0;
 		if (!hesc_attr(ob, date, strlen(date)))
 			return 0;
@@ -1142,33 +1208,64 @@ rndr_doc_header(struct lowdown_buf *ob,
 	}
 
 	if (!rndr_meta_multi(ob, css, 1,
-	    "<link rel=\"stylesheet\" href=", " />"))
+	    "<link rel=\"stylesheet\" href=\"", "\" />"))
+		return 0;
+	if (!rndr_meta_multi(ob, script, 1,
+	     "<script src=\"", "\"></script>"))
 		return 0;
 
-	if (!rndr_meta_multi(ob, script, 1,
-	     "<script src=", "></script>"))
-		return 0;
+	/* In HTML5, the title is required. */
 
 	if (!HBUF_PUTSL(ob, "<title>"))
 		return 0;
-	if (!hesc_html(ob, title, strlen(title), 
-	    st->flags & LOWDOWN_HTML_OWASP, 0,
-	    st->flags & LOWDOWN_HTML_NUM_ENT))
+	if (title != NULL &&
+	    !hesc_html(ob, title, strlen(title),
+		    st->flags & LOWDOWN_HTML_OWASP, 0,
+		    st->flags & LOWDOWN_HTML_NUM_ENT))
 		return 0;
-	if (!HBUF_PUTSL(ob, "</title>\n"))
+	if (!HBUF_PUTSL(ob, "</title>\n</head>\n<body>\n"))
 		return 0;
-	return HBUF_PUTSL(ob, "</head>\n<body>\n");
+
+	if (!(st->flags & LOWDOWN_HTML_TITLEBLOCK))
+		return 1;
+	if (author == NULL && title == NULL && date == NULL)
+		return 1;
+
+	/*
+	 * Output a title block.  This will be empty if we don't have
+	 * any title components to output.
+	 */
+
+	if (!HBUF_PUTSL(ob, "<header id=\"title-block-header\">\n"))
+		return 0;
+	if (title != NULL &&
+	    (!HBUF_PUTSL(ob, "<h1 class=\"title\">") ||
+	     !hesc_html(ob, title, strlen(title), 
+		     st->flags & LOWDOWN_HTML_OWASP, 0,
+		     st->flags & LOWDOWN_HTML_NUM_ENT) ||
+	     !HBUF_PUTSL(ob, "</h1>\n")))
+			return 0;
+	if (author != NULL &&
+	    !rndr_meta_multi(ob, author, 0,
+		    "<p class=\"author\">", "</p>"))
+		return 0;
+	if (date != NULL &&
+	    (!HBUF_PUTSL(ob, "<p class=\"date\">") ||
+	     !hesc_html(ob, date, strlen(date), 
+		     st->flags & LOWDOWN_HTML_OWASP, 0,
+		     st->flags & LOWDOWN_HTML_NUM_ENT) ||
+	     !HBUF_PUTSL(ob, "</p>\n")))
+			return 0;
+	return HBUF_PUTSL(ob, "</header>\n");
 }
 
 static int
-rndr(struct lowdown_buf *ob,
-	struct lowdown_metaq *mq, void *ref, 
+rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq, struct html *st,
 	const struct lowdown_node *n)
 {
 	const struct lowdown_node	*child;
 	struct lowdown_buf		*tmp;
 	int32_t				 ent;
-	struct html			*st = ref;
 	int				 ret = 1, rc = 1;
 
 	if ((tmp = hbuf_new(64)) == NULL)
@@ -1208,7 +1305,7 @@ rndr(struct lowdown_buf *ob,
 		rc = rndr_blockcode(ob, &n->rndr_blockcode, st);
 		break;
 	case LOWDOWN_BLOCKQUOTE:
-		rc = rndr_blockquote(ob, tmp);
+		rc = rndr_blockquote(st, &n->rndr_blockquote, ob, tmp);
 		break;
 	case LOWDOWN_DEFINITION:
 		rc = rndr_definition(ob, tmp);
@@ -1291,7 +1388,10 @@ rndr(struct lowdown_buf *ob,
 		rc = rndr_strikethrough(ob, tmp);
 		break;
 	case LOWDOWN_SUPERSCRIPT:
-		rc = rndr_superscript(ob, tmp);
+		rc = rndr_superscript(ob, tmp, n->type);
+		break;
+	case LOWDOWN_SUBSCRIPT:
+		rc = rndr_superscript(ob, tmp, n->type);
 		break;
 	case LOWDOWN_FOOTNOTE:
 		rc = rndr_footnote_ref(ob, tmp, st);
