@@ -1,6 +1,5 @@
-/*	$Id$ */
 /*
- * Copyright (c) 2016, 2017, 2020 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -235,7 +234,8 @@ main(int argc, char *argv[])
 	int			 c, diff = 0, fd, status = 0, afl = 0,
 				 rfl = 0, aifl = 0, rifl = 0,
 				 centre = 0, list = 0;
-	char			*ret = NULL, *cp, *odtsty = NULL;
+	char			*ret = NULL, *cp, *odtsty = NULL,
+				*nroffcodefn = NULL;
 	size_t		 	 i, retsz = 0, rcols, sz;
 	ssize_t			 ssz;
 	struct lowdown_meta 	*m;
@@ -255,6 +255,10 @@ main(int argc, char *argv[])
 		{ "html-no-skiphtml",	no_argument,	&rfl, LOWDOWN_HTML_SKIP_HTML },
 		{ "html-titleblock",	no_argument,	&afl, LOWDOWN_HTML_TITLEBLOCK },
 		{ "html-no-titleblock",	no_argument,	&rfl, LOWDOWN_HTML_TITLEBLOCK },
+		{ "html-callout-mdn",	no_argument,	&afl, LOWDOWN_HTML_CALLOUT_MDN },
+		{ "html-no-callout-mdn",no_argument,	&rfl, LOWDOWN_HTML_CALLOUT_MDN },
+		{ "html-callout-gfm",	no_argument,	&afl, LOWDOWN_HTML_CALLOUT_GFM },
+		{ "html-no-callout-gfm",no_argument,	&rfl, LOWDOWN_HTML_CALLOUT_GFM },
 
 		{ "latex-numbered",	no_argument,	&afl, LOWDOWN_LATEX_NUMBERED },
 		{ "latex-no-numbered",	no_argument,	&rfl, LOWDOWN_LATEX_NUMBERED },
@@ -271,6 +275,7 @@ main(int argc, char *argv[])
 		{ "nroff-no-shortlinks",no_argument, 	&rfl, LOWDOWN_NROFF_SHORTLINK },
 		{ "nroff-skiphtml",	no_argument,	&afl, LOWDOWN_NROFF_SKIP_HTML },
 		{ "nroff-no-skiphtml",	no_argument,	&rfl, LOWDOWN_NROFF_SKIP_HTML },
+		{ "nroff-code-font",	required_argument, NULL, 7 },
 
 		{ "odt-skiphtml",	no_argument,	&afl, LOWDOWN_ODT_SKIP_HTML },
 		{ "odt-no-skiphtml",	no_argument,	&rfl, LOWDOWN_ODT_SKIP_HTML },
@@ -300,6 +305,8 @@ main(int argc, char *argv[])
 		{ "term-no-nolinks",	no_argument, 	&rfl, LOWDOWN_TERM_NOLINK },
 		{ "term-shortlinks",	no_argument, 	&afl, LOWDOWN_TERM_SHORTLINK },
 		{ "term-no-shortlinks",	no_argument, 	&rfl, LOWDOWN_TERM_SHORTLINK },
+		{ "term-all-meta",	no_argument, 	&afl, LOWDOWN_TERM_ALL_META },
+		{ "term-no-all-meta",	no_argument, 	&rfl, LOWDOWN_TERM_ALL_META },
 
 		{ "out-smarty",		no_argument,	&afl, LOWDOWN_SMARTY },
 		{ "out-no-smarty",	no_argument,	&rfl, LOWDOWN_SMARTY },
@@ -320,6 +327,8 @@ main(int argc, char *argv[])
 		{ "parse-no-strike",	no_argument,	&rifl, LOWDOWN_STRIKE },
 		{ "parse-super",	no_argument,	&aifl, LOWDOWN_SUPER },
 		{ "parse-no-super",	no_argument,	&rifl, LOWDOWN_SUPER },
+		{ "parse-super-short",	no_argument,	&aifl, LOWDOWN_SUPER_SHORT },
+		{ "parse-no-super-short",no_argument,	&rifl, LOWDOWN_SUPER_SHORT },
 		{ "parse-math",		no_argument,	&aifl, LOWDOWN_MATH },
 		{ "parse-no-math",	no_argument,	&rifl, LOWDOWN_MATH },
 		{ "parse-mantitle",	no_argument,	&aifl, LOWDOWN_MANTITLE },
@@ -341,6 +350,8 @@ main(int argc, char *argv[])
 		{ "parse-no-ext-attrs",	no_argument,	&rifl, LOWDOWN_ATTRS },
 		{ "parse-tasklists",	no_argument,	&aifl, LOWDOWN_TASKLIST },
 		{ "parse-no-tasklists",	no_argument,	&rifl, LOWDOWN_TASKLIST },
+		{ "parse-callouts",	no_argument,	&aifl, LOWDOWN_CALLOUTS },
+		{ "parse-no-callouts",	no_argument,	&rifl, LOWDOWN_CALLOUTS },
 		{ "parse-maxdepth",	required_argument, NULL, 5 },
 
 		{ NULL,			0,	NULL,	0 }
@@ -364,6 +375,7 @@ main(int argc, char *argv[])
 		LOWDOWN_DEFLIST |
 		LOWDOWN_FENCED |
 		LOWDOWN_FOOTNOTES |
+		LOWDOWN_CALLOUTS |
 		LOWDOWN_MANTITLE |
 		LOWDOWN_METADATA |
 		LOWDOWN_STRIKE |
@@ -480,6 +492,22 @@ main(int argc, char *argv[])
 		case 6:
 			odtstyfn = optarg;
 			break;
+		case 7:
+			/*
+			 * Break down some aliases here: "none", "bold",
+			 * or "code".
+			 */
+			if (strcmp(optarg, "none") == 0)
+				nroffcodefn = strdup("R,B,I,BI");
+			else if (strcmp(optarg, "bold") == 0)
+				nroffcodefn = strdup("B,B,BI,BI");
+			else if (strcmp(optarg, "code") == 0)
+				nroffcodefn = strdup("CR,CB,CI,CBI");
+			else
+				nroffcodefn = strdup(optarg);
+			if (nroffcodefn == NULL)
+				err(1, NULL);
+			break;
 		default:
 			goto usage;
 		}
@@ -561,6 +589,48 @@ main(int argc, char *argv[])
 		opts.odt.sty = odtsty;
 	}
 
+	/*
+	 * If specified and in -tman or -tms, parse the constant width
+	 * fonts.  As mentioned in nroff.c, the code font "C" is not
+	 * portable, so let the user override it.  This comes as a
+	 * comma-separated sequence of R[,B[,I[,BI]]].  Any of these may
+	 * inherit the default by being unspecified or empty, e.g.,
+	 * "CR,CB".
+	 */
+
+	if ((opts.type == LOWDOWN_MAN || opts.type == LOWDOWN_NROFF) &&
+	    nroffcodefn != NULL && *nroffcodefn != '\0') {
+		opts.nroff.cr = cp = nroffcodefn;
+		while (*cp != '\0' && *cp != ',')
+			cp++;
+		if (*cp != '\0') {
+			*cp++ = '\0';
+			opts.nroff.cb = cp;
+			while (*cp != '\0' && *cp != ',')
+				cp++;
+			if (*cp != '\0') {
+				*cp++ = '\0';
+				opts.nroff.ci = cp;
+				while (*cp != '\0' && *cp != ',')
+					cp++;
+				if (*cp != '\0') {
+					*cp++ = '\0';
+					opts.nroff.cbi = cp;
+					while (*cp != '\0' && *cp != ',')
+						cp++;
+				}
+			}
+		}
+		if (opts.nroff.cr != NULL && *opts.nroff.cr == '\0')
+			opts.nroff.cr = NULL;
+		if (opts.nroff.cb != NULL && *opts.nroff.cb == '\0')
+			opts.nroff.cb = NULL;
+		if (opts.nroff.ci != NULL && *opts.nroff.ci == '\0')
+			opts.nroff.ci = NULL;
+		if (opts.nroff.cbi != NULL && *opts.nroff.cbi == '\0')
+			opts.nroff.cbi = NULL;
+	}
+
 	/* Configure the output file. */
 
 	if (fnout != NULL && strcmp(fnout, "-") &&
@@ -616,6 +686,7 @@ main(int argc, char *argv[])
 
 	free(ret);
 	free(odtsty);
+	free(nroffcodefn);
 
 	if (fout != stdout)
 		fclose(fout);
