@@ -1,6 +1,5 @@
-/*	$Id$ */
 /*
- * Copyright (c) 2017--2021 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -58,6 +57,7 @@ static	const char *const names[LOWDOWN__MAX] = {
 	"LOWDOWN_LINK",                 /* LOWDOWN_LINK */
 	"LOWDOWN_TRIPLE_EMPHASIS",      /* LOWDOWN_TRIPLE_EMPHASIS */
 	"LOWDOWN_STRIKETHROUGH",        /* LOWDOWN_STRIKETHROUGH */
+	"LOWDOWN_SUBSCRIPT",		/* LOWDOWN_SUBSCRIPT */
 	"LOWDOWN_SUPERSCRIPT",          /* LOWDOWN_SUPERSCRIPT */
 	"LOWDOWN_FOOTNOTE",		/* LOWDOWN_FOOTNOTE */
 	"LOWDOWN_MATH_BLOCK",           /* LOWDOWN_MATH_BLOCK */
@@ -273,6 +273,20 @@ rndr(struct lowdown_buf *ob,
 		if (!HBUF_PUTSL(ob, "\n"))
 			return 0;
 		break;
+	case LOWDOWN_BLOCKQUOTE:
+		if (root->rndr_blockquote.type == BLOCKQUOTE_REGULAR)
+			break;
+		if (!rndr_indent(ob, indent + 1))
+			return 0;
+		if (!hbuf_printf(ob, "admonition (%s): %s\n",
+		    root->rndr_blockquote.type == BLOCKQUOTE_ADMONITION ?
+		    "single-line" : "double-line",
+		    root->rndr_blockquote.admonition == ADMONITION_NOTE ?
+		    "note" :
+		    root->rndr_blockquote.admonition == ADMONITION_WARNING ?
+		    "warning" : "callout"))
+			return 0;
+		break;
 	case LOWDOWN_BLOCKCODE:
 		if (!rndr_indent(ob, indent + 1))
 			return 0;
